@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.21 2004/11/05 11:51:30 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.22 2004/11/12 11:45:40 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -228,6 +228,7 @@ int main(int argc, char **argv)
 	signal(SIGQUIT, exit_handler);
 	signal(SIGCHLD, reap_handler);
 	signal(SIGUSR1, failover_handler);
+	signal(SIGUSR2, failover_handler);
 
 	for (;;)
 	{
@@ -518,13 +519,18 @@ static void myexit(int code)
 	exit(code);
 }
 
-/* notice backend connection error using SIGUSR1 */
-void notice_backend_error(void)
+/* notice backend connection error using SIGUSR1 or SIGUSR2 */
+void notice_backend_error(int master)
 {
 	pid_t parent = getppid();
 
-	pool_log("notice_backend_error: fail over request from pid %d", getpid());
-	kill(parent, SIGUSR1);
+	pool_log("notice_backend_error: master: %d fail over request from pid %d", master, getpid());
+
+	if (master)
+		kill(parent, SIGUSR1);
+	else
+		kill(parent, SIGUSR2);
+
 	/* avoid race conditon with SIGCHLD */
 	sleep(1);
 }
@@ -566,12 +572,16 @@ static RETSIGTYPE exit_handler(int sig)
 
 
 /*
- * handle SIGUSR1 (backend connection error, fail over request, if possible)
+ * handle SIGUSR1/SIGUSR2 (backend connection error, fail over request, if possible)
+ *
+ * if sig == SIGUSR1, we assume that the master has been down.
+ * if sig == SIGUSR2, we assume that the secondary has been down.
  */
 static RETSIGTYPE failover_handler(int sig)
 {
 	int i;
 	int replication = 0;
+	static int degenerated = 0;	/* set non 0 if already degerated */
 
 	pool_debug("failover_handler called");
 
@@ -606,11 +616,22 @@ static RETSIGTYPE failover_handler(int sig)
 		if (pool_config.replication_enabled)
 		{
 			replication = 1;
-			pool_log("starting degenration. shutdown secondary host %s(%d)",
-					   pool_config.secondary_backend_host_name,
-					   pool_config.secondary_backend_port);
+			degenerated = 1;
+
+			if (sig == SIGUSR2)
+			{
+				pool_log("starting degeneration. shutdown secondary host %s(%d)",
+						 pool_config.secondary_backend_host_name,
+						 pool_config.secondary_backend_port);
+			}
+			else
+			{
+				pool_log("starting degeneration. shutdown master host %s(%d)",
+						 pool_config.backend_host_name,
+						 pool_config.backend_port);
+			}
 		}
-		else if (pool_config.secondary_backend_port != 0)
+		else if (!degenerated && pool_config.secondary_backend_port != 0)
 		{
 			pool_log("starting failover from %s(%d) to %s(%d)",
 					   pool_config.current_backend_host_name,
@@ -640,8 +661,14 @@ static RETSIGTYPE failover_handler(int sig)
 		{
 			/* disable replicaton mode */
 			pool_config.replication_enabled = 0;
+
+			if (sig == SIGUSR1)
+			{
+				pool_config.current_backend_host_name = pool_config.secondary_backend_host_name;
+				pool_config.current_backend_port = pool_config.secondary_backend_port;
+			}
 		}
-		else if (pool_config.secondary_backend_port != 0)
+		else if (!degenerated && pool_config.secondary_backend_port != 0)
 		{
 			/* fail over to secondary */
 			pool_config.current_backend_host_name = pool_config.secondary_backend_host_name;
@@ -659,15 +686,22 @@ static RETSIGTYPE failover_handler(int sig)
 		 * need to fork a new child which should inherit these fds.
 		 */
 
-		switching = 0;
-
 		if (replication)
 		{
-			pool_log("degenration done. shutdown secondary host %s(%d)",
-					   pool_config.secondary_backend_host_name,
-					   pool_config.secondary_backend_port);
+			if (sig == SIGUSR2)
+			{
+				pool_log("degenration done. shutdown secondary host %s(%d)",
+						 pool_config.secondary_backend_host_name,
+						 pool_config.secondary_backend_port);
+			}
+			else
+			{
+				pool_log("degeneration done. shutdown master host %s(%d)",
+						 pool_config.backend_host_name,
+						 pool_config.backend_port);
+			}
 		}
-		else if (pool_config.secondary_backend_port != 0)
+		else if (!degenerated && pool_config.secondary_backend_port != 0)
 		{
 			pool_log("failover from %s(%d) to %s(%d) done.",
 					   pool_config.backend_host_name,
@@ -679,6 +713,8 @@ static RETSIGTYPE failover_handler(int sig)
 		{
 			pool_log("restarting pgpool done.");
 		}
+
+		switching = 0;
 	}
 }
 
@@ -705,7 +741,7 @@ static RETSIGTYPE reap_handler(int sig)
 		{
 #endif
 
-			pool_debug("child %d exits with signal %d", pid, WTERMSIG(status));
+			pool_debug("child %d exits with status %d by signal %d", pid, status, WTERMSIG(status));
 
 			/* look for exiting child's pid */
 			for (i=0;i<pool_config.num_init_children;i++)
@@ -713,7 +749,7 @@ static RETSIGTYPE reap_handler(int sig)
 				if (pid == pids[i])
 				{
 					/* if found, fork a new child */
-					if (!switching && !exiting)
+					if (!switching && !exiting && status)
 					{
 						pids[i] = fork_a_child(unix_fd, inet_fd);
 						break;
