@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.34 2004/10/28 03:54:03 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.36 2004/11/12 12:03:19 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -1535,7 +1535,7 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 
 	status[i].name = "child_life_time";
 	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.child_life_time);
-	status[i].desc = "if idle for this seconds, child exits (not implemented yet)";
+	status[i].desc = "if idle for this seconds, child exits";
 	i++;
 
 	status[i].name = "connection_life_time";
@@ -1770,7 +1770,7 @@ void pool_send_frontend_exits(POOL_CONNECTION_POOL *backend)
 		if (MAJOR(backend) == PROTO_MAJOR_V3)
 		{
 			len = htonl(4);
-			pool_write(MASTER(backend), &len, sizeof(len));
+			pool_write(SECONDARY(backend), &len, sizeof(len));
 		}
 		fflush(SECONDARY(backend)->write_fd);
 	}
